CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

OBJ
  c : "clib"

PUB start
  c.start
  main
{****************************************************************************
* file:         micro.c
* abstract:     This file contains the function, xsvfExecute(),
*               call for interpreting the XSVF commands.
* Usage:        Call xsvfExecute() to process XSVF data.
*               The XSVF data is retrieved by readByte() in ports.c
*               Remove the main function if you already have one.
* Options:      XSVF_SUPPORT_COMPRESSION
*                   This define supports the XC9500/XL compression scheme.
*                   This define adds support for XSDRINC and XSETSDRMASKS.
*               XSVF_SUPPORT_ERRORCODES
*                   This define causes the xsvfExecute function to return
*                   an error code for specific errors.  See error codes below.
*                   If this is not defined, the return value defaults to the
*                   legacy values for backward compatibility:
*                   1 = success;  0 = failure.
* Debugging:    DEBUG_MODE (Legacy name)
*               Define DEBUG_MODE to compile with debugging features.
*               Both micro.c and ports.c must be compiled with the DEBUG_MODE
*               defined to enable the standalone main implementation in
*               micro.c that reads XSVF from a file.
* History:      v2.00   - Original XSVF implementation.
*               v4.04   - Added delay at end of XSIR for XC18v00 support.
*                         Added new commands for CoolRunner support:
*                         XSTATE, XENDIR, XENDDR
*               v4.05   - Cleanup micro.c but leave ports.c intact.
*               v4.06   - Fix xsvfGotoTapState for retry transition.
*               v4.07   - Update example waitTime implementations for
*                         compatibility with Virtex-II.
*               v4.10   - Add new XSIR2 command that supports a 2-byte
*                         IR-length parameter for IR shifts > 255 bits.
*               v4.11   - No change.  Update version to match SVF2XSVF xlator.
*               v4.14   - Added XCOMMENT.
*               v5.00   - Improve XSTATE support.
*                         Added XWAIT.
*               v5.01   - make sure that TCK is low during RUNTEST wait for
*                         XC18V00/XCF00 support.  Only change is in PORTS.C
*                         waitTime() function for implementations that do NOT
*                         pulse TCK during the waitTime.
****************************************************************************}
{============================================================================
* #pragmas
============================================================================}
{============================================================================
* #include files
============================================================================}
'CTOSPIN WARNING #include <stdio.h>
'CTOSPIN WARNING #include <stdlib.h>
'CTOSPIN WARNING #include <string.h>
'CTOSPIN WARNING #include <time.h>
{ DEBUG_MODE }
{****************************************************************************
* File:         micro.h
* Description:  This header file contains the function prototype to the
*               primary interface function for the XSVF player.
* Usage:        FIRST - PORTS.C
*               Customize the ports.c function implementations to establish
*               the correct protocol for communicating with your JTAG ports
*               (setPort() and readTDOBit()) and tune the waitTime() delay
*               function.  Also, establish access to the XSVF data source
*               in the readByte() function.
*               FINALLY - Call xsvfExecute().
****************************************************************************}
{ Legacy error codes for xsvfExecute from original XSVF player v2.0 }

CON
  XSVF_LEGACY_SUCCESS = 1
  XSVF_LEGACY_ERROR = 0
{ 4.04 [NEW] Error codes for xsvfExecute. }
{ Must #define XSVF_SUPPORT_ERRORCODES in micro.c to get these codes }
  XSVF_ERROR_NONE = 0
  XSVF_ERROR_UNKNOWN = 1
  XSVF_ERROR_TDOMISMATCH = 2
  XSVF_ERROR_MAXRETRIES = 3 { TDO mismatch after max retries }
  XSVF_ERROR_ILLEGALCMD = 4
  XSVF_ERROR_ILLEGALSTATE = 5
  XSVF_ERROR_DATAOVERFLOW = 6 { Data > lenVal MAX_LEN buffer size}
{ Insert new errors here }
  XSVF_ERROR_LAST = 7
{****************************************************************************
* Function:     xsvfExecute
* Description:  Process, interpret, and apply the XSVF commands.
*               See port.c:readByte for source of XSVF data.
* Parameters:   none.
* Returns:      int - For error codes see above.
****************************************************************************}
{ XSVF_MICRO_H }
{*****************************************************}
{ file: lenval.h                                      }
{ abstract:  This file contains a description of the  }
{            data structure "lenval".                 }
{*****************************************************}
{ the lenVal structure is a byte oriented type used to store an }
{ arbitrary length binary value. As an example, the hex value   }
{ 0x0e3d is represented as a lenVal with len=2 (since 2 bytes   }
{ and val[0]=0e and val[1]=3d.  val[2-MAX_LEN] are undefined    }
{ maximum length (in bytes) of value to read in        }
{ this needs to be at least 4, and longer than the     }
{ length of the longest SDR instruction.  If there is, }
{ only 1 device in the chain, MAX_LEN must be at least }
{ ceil(27/8) == 4.  For 6 devices in a chain, MAX_LEN  }
{ must be 5, for 14 devices MAX_LEN must be 6, for 20  }
{ devices MAX_LEN must be 7, etc..                     }
{ You can safely set MAX_LEN to a smaller number if you}
{ know how many devices will be in your chain.         }
{  #define MAX_LEN (Actual #define is below this comment block)
      This #define defines the maximum length (in bytes) of predefined
      buffers in which the XSVF player stores the current shift data.
      This length must be greater than the longest shift length (in bytes)
      in the XSVF files that will be processed.  7000 is a very conservative
      number.  The buffers are stored on the stack and if you have limited
      stack space, you may decrease the MAX_LEN value.
      How to find the "shift length" in bits?
      Look at the ASCII version of the XSVF (generated with the -a option
      for the SVF2XSVF translator) and search for the XSDRSIZE command
      with the biggest parameter.  XSDRSIZE is equivalent to the SVF's
      SDR length plus the lengths of applicable HDR and TDR commands.
      Remember that the MAX_LEN is defined in bytes.  Therefore, the
      minimum MAX_LEN = ceil( max( XSDRSIZE ) / 8 );
      The following MAX_LEN values have been tested and provide relatively
      good margin for the corresponding devices:
        DEVICE       MAX_LEN   Resulting Shift Length Max (in bits)
        ---------    -------   ----------------------------------------------
        XC9500/XL/XV 32        256
        CoolRunner/II 256      2048   - actual max 1 device = 1035 bits
        FPGA         128       1024    - svf2xsvf -rlen 1024
        XC18V00/XCF00
                     1100      8800   - no blank check performed (default)
                                      - actual max 1 device = 8192 bits verify
                                      - max 1 device = 4096 bits program-only
        XC18V00/XCF00 when using the optional Blank Check operation
                     2500      20000  - required for blank check
                                      - blank check max 1 device = 16384 bits
}
  MAX_LEN = 2500
' typedef struct var_len_byte
  len = 0 { number of chars in this value }
  val = 2 { bytes of data }
{ return the long representation of a lenVal }
{ set lenVal equal to value }
{ check if expected equals actual (taking the mask into account) }
{ add val1+val2 and put the result in resVal }
{ return the (byte, bit) of lv (reading from left to right) }
{ set the (byte, bit) of lv equal to val (e.g. SetBit("00000000",byte, 1)
   equals "01000000" }
{ read from XSVF numBytes bytes of data into x }
{*****************************************************}
{ file: ports.h                                       }
{ abstract:  This file contains extern declarations   }
{            for providing stimulus to the JTAG ports.}
{*****************************************************}
{ these constants are used to send the appropriate ports to setPort }
{ they should be enumerated types, but some of the microcontroller  }
{ compilers don't like enumerated types }
  TCK = 0
  TMS = 1
  TDI = 2
{ set the port "p" (TCK, TMS, or TDI) to val (0 or 1) }
{ read the TDO bit and store it in val }
{ make clock go down->up->down}
{ read the next byte of data from the xsvf file }
{============================================================================
* XSVF #define
============================================================================}
{****************************************************************************
* Define:       XSVF_SUPPORT_COMPRESSION
* Description:  Define this to support the XC9500/XL XSVF data compression
*               scheme.
*               Code size can be reduced by NOT supporting this feature.
*               However, you must use the -nc (no compress) option when
*               translating SVF to XSVF using the SVF2XSVF translator.
*               Corresponding, uncompressed XSVF may be larger.
****************************************************************************}
  XSVF_SUPPORT_COMPRESSION = 1
{****************************************************************************
* Define:       XSVF_SUPPORT_ERRORCODES
* Description:  Define this to support the new XSVF error codes.
*               (The original XSVF player just returned 1 for success and
*               0 for an unspecified failure.)
****************************************************************************}
  XSVF_SUPPORT_ERRORCODES = 1

PUB XSVF_ERRORCODE(errorCode)
  RETURN errorCode
{****************************************************************************
* Define:       XSVF_MAIN
* Description:  Define this to compile with a main function for standalone
*               debugging.
****************************************************************************}

CON
  XSVF_MAIN = 1
{ DEBUG_MODE }
{ XSVF_MAIN }
{============================================================================
* DEBUG_MODE #define
============================================================================}

VAR
  LONG xsvf_iDebugLevel

PUB XSVFDBG_PRINTF(iDebugLevel, pzFormat)
  IF (xsvf_iDebugLevel => iDebugLevel)
    c.printf0(pzFormat)

PUB XSVFDBG_PRINTF1(iDebugLevel, pzFormat, arg1)
  IF (xsvf_iDebugLevel => iDebugLevel)
    c.printf1(pzFormat, arg1)

PUB XSVFDBG_PRINTF2(iDebugLevel, pzFormat, arg1, arg2)
  IF (xsvf_iDebugLevel => iDebugLevel)
    c.printf2(pzFormat, arg1, arg2)

PUB XSVFDBG_PRINTF3(iDebugLevel, pzFormat, arg1, arg2, arg3)
  IF (xsvf_iDebugLevel => iDebugLevel)
    c.printf3(pzFormat, arg1, arg2, arg3)

PUB XSVFDBG_PRINTLENVAL(iDebugLevel, plenVal)
  IF (xsvf_iDebugLevel => iDebugLevel)
    xsvfPrintLenVal(plenVal)
{============================================================================
* XSVF Type Declarations
============================================================================}
{****************************************************************************
* Struct:       SXsvfInfo
* Description:  This structure contains all of the data used during the
*               execution of the XSVF.  Some data is persistent, predefined
*               information (e.g. lRunTestTime).  The bulk of this struct's
*               size is due to the lenVal structs (defined in lenval.h)
*               which contain buffers for the active shift data.  The MAX_LEN
*               #define in lenval.h defines the size of these buffers.
*               These buffers must be large enough to store the longest
*               shift data in your XSVF file.  For example:
*                   MAX_LEN >= ( longest_shift_data_in_bits / 8 )
*               Because the lenVal struct dominates the space usage of this
*               struct, the rough size of this struct is:
*                   sizeof( SXsvfInfo ) ~= MAX_LEN * 7 (number of lenVals)
*               xsvfInitialize() contains initialization code for the data
*               in this struct.
*               xsvfCleanup() contains cleanup code for the data in this
*               struct.
****************************************************************************}

CON
' typedef struct tagSXsvfInfo
  { XSVF status information }
  ucComplete = 0 { 0 = running; 1 = complete }
  ucCommand = 1 { Current XSVF command byte }
  lCommandCount = 4 { Number of commands processed }
  iErrorCode = 8 { An error code. 0 = no error. }
  { TAP state/sequencing information }
  ucTapState = 12 { Current TAP state }
  ucEndIR = 13 { ENDIR TAP state (See SVF) }
  ucEndDR = 14 { ENDDR TAP state (See SVF) }
  { RUNTEST information }
  ucMaxRepeat = 15 { Max repeat loops (for xc9500/xl) }
  lRunTestTime = 16 { Pre-specified RUNTEST time (usec) }
  { Shift Data Info and Buffers }
  lShiftLengthBits = 20 { Len. current shift data in bits }
  sShiftLengthBytes = 24 { Len. current shift data in bytes }
  lvTdi = 26 { Current TDI shift data }
  lvTdoExpected = 2530 { Expected TDO shift data }
  lvTdoCaptured = 5034 { Captured TDO shift data }
  lvTdoMask = 7538 { TDO mask: 0=dontcare; 1=compare }
  { XSDRINC Data Buffers }
  lvAddressMask = 10042 { Address mask for XSDRINC }
  lvDataMask = 12546 { Data mask for XSDRINC }
  lvNextData = 15050 { Next data for XSDRINC }
  { XSVF_SUPPORT_COMPRESSION }
{============================================================================
* XSVF Command Bytes
============================================================================}
{ encodings of xsvf instructions }
  XCOMPLETE = 0
  XTDOMASK = 1
  XSIR = 2
  XSDR = 3
  XRUNTEST = 4
{ Reserved              5 }
{ Reserved              6 }
  XREPEAT = 7
  XSDRSIZE = 8
  XSDRTDO = 9
  XSETSDRMASKS = 10
  XSDRINC = 11
  XSDRB = 12
  XSDRC = 13
  XSDRE = 14
  XSDRTDOB = 15
  XSDRTDOC = 16
  XSDRTDOE = 17
  XSTATE = 18 { 4.00 }
  XENDIR = 19 { 4.04 }
  XENDDR = 20 { 4.04 }
  XSIR2 = 21 { 4.10 }
  XCOMMENT = 22 { 4.14 }
  XWAIT = 23 { 5.00 }
{ Insert new commands here }
{ and add corresponding xsvfDoCmd function to xsvf_pfDoCmd below. }
  XLASTCMD = 24 { Last command marker }
{============================================================================
* XSVF Command Parameter Values
============================================================================}
  XSTATE_RESET = 0 { 4.00 parameter for XSTATE }
  XSTATE_RUNTEST = 1 { 4.00 parameter for XSTATE }
  XENDXR_RUNTEST = 0 { 4.04 parameter for XENDIR/DR }
  XENDXR_PAUSE = 1 { 4.04 parameter for XENDIR/DR }
{ TAP states }
  XTAPSTATE_RESET = 0
  XTAPSTATE_RUNTEST = 1 { a.k.a. IDLE }
  XTAPSTATE_SELECTDR = 2
  XTAPSTATE_CAPTUREDR = 3
  XTAPSTATE_SHIFTDR = 4
  XTAPSTATE_EXIT1DR = 5
  XTAPSTATE_PAUSEDR = 6
  XTAPSTATE_EXIT2DR = 7
  XTAPSTATE_UPDATEDR = 8
  XTAPSTATE_IRSTATES = 9 { All IR states begin here }
  XTAPSTATE_SELECTIR = 9
  XTAPSTATE_CAPTUREIR = 10
  XTAPSTATE_SHIFTIR = 11
  XTAPSTATE_EXIT1IR = 12
  XTAPSTATE_PAUSEIR = 13
  XTAPSTATE_EXIT2IR = 14
  XTAPSTATE_UPDATEIR = 15
{============================================================================
* XSVF Function Prototypes
============================================================================}
{ Illegal command function }
{ Insert new command functions here }
{============================================================================
* XSVF Global Variables
============================================================================}
{ Array of XSVF command functions.  Must follow command byte value order! }
{ If your compiler cannot take this form, then convert to a switch statement}

PUB xsvf_DoCmd(command, pXsvfInfo) | retval
  CASE(command)
    0 :
      retval := xsvfDoXCOMPLETE(pXsvfInfo)
    1 :
      retval := xsvfDoXTDOMASK(pXsvfInfo)
    2 :
      retval := xsvfDoXSIR(pXsvfInfo)
    3 :
      retval := xsvfDoXSDR(pXsvfInfo)
    4 :
      retval := xsvfDoXRUNTEST(pXsvfInfo)
    5 :
      retval := xsvfDoIllegalCmd(pXsvfInfo)
    6 :
      retval := xsvfDoIllegalCmd(pXsvfInfo)
    7 :
      retval := xsvfDoXREPEAT(pXsvfInfo)
    8 :
      retval := xsvfDoXSDRSIZE(pXsvfInfo)
    9 :
      retval := xsvfDoXSDRTDO(pXsvfInfo)
    10 :
      retval := xsvfDoXSETSDRMASKS(pXsvfInfo)
    11 :
      retval := xsvfDoXSDRINC(pXsvfInfo)
    12 :
      retval := xsvfDoXSDRBCE(pXsvfInfo)
    13 :
      retval := xsvfDoXSDRBCE(pXsvfInfo)
    14 :
      retval := xsvfDoXSDRBCE(pXsvfInfo)
    15 :
      retval := xsvfDoXSDRTDOBCE(pXsvfInfo)
    16 :
      retval := xsvfDoXSDRTDOBCE(pXsvfInfo)
    17 :
      retval := xsvfDoXSDRTDOBCE(pXsvfInfo)
    18 :
      retval := xsvfDoXSTATE(pXsvfInfo)
    19 :
      retval := xsvfDoXENDXR(pXsvfInfo)
    20 :
      retval := xsvfDoXENDXR(pXsvfInfo)
    21 :
      retval := xsvfDoXSIR2(pXsvfInfo)
    22 :
      retval := xsvfDoXCOMMENT(pXsvfInfo)
    23 :
      retval := xsvfDoXWAIT(pXsvfInfo)
    OTHER :
  RETURN retval

DAT
  datstr BYTE "XCOMPLETE", 0, "XTDOMASK", 0, "XSIR", 0
  BYTE "XSDR", 0, "XRUNTEST", 0, "Reserved5", 0, "Reserved6", 0
  BYTE "XREPEAT", 0, "XSDRSIZE", 0, "XSDRTDO", 0, "XSETSDRMASKS", 0
  BYTE "XSDRINC", 0, "XSDRB", 0, "XSDRC", 0, "XSDRE", 0
  BYTE "XSDRTDOB", 0, "XSDRTDOC", 0, "XSDRTDOE", 0, "XSTATE", 0
  BYTE "XENDIR", 0, "XENDDR", 0, "XSIR2", 0, "XCOMMENT", 0
  BYTE "XWAIT", 0
  xsvf_pzCommandName LONG @datstr + 16, @datstr + 26, @datstr + 35
  LONG @datstr + 40, @datstr + 45, @datstr + 54, @datstr + 64
  LONG @datstr + 74, @datstr + 82, @datstr + 91, @datstr + 99
  LONG @datstr + 112, @datstr + 120, @datstr + 126, @datstr + 132
  LONG @datstr + 138, @datstr + 147, @datstr + 156, @datstr + 165
  LONG @datstr + 172, @datstr + 179, @datstr + 186, @datstr + 192
  LONG @datstr + 201
  datstr_0 BYTE "No error", 0, "ERROR:  Unknown", 0
  BYTE "ERROR:  TDO mismatch", 0, "ERROR:  TDO mismatch and exceeded max retries", 0
  BYTE "ERROR:  Unsupported XSVF command", 0, "ERROR:  Illegal state specification", 0
  BYTE "ERROR:  Data overflows allocated MAX_LEN buffer size", 0
  xsvf_pzErrorName LONG @datstr_0 + 16, @datstr_0 + 25, @datstr_0 + 41
  LONG @datstr_0 + 62, @datstr_0 + 108, @datstr_0 + 141, @datstr_0 + 177
  datstr_1 BYTE "RESET", 0, "RUNTEST/IDLE", 0, "DRSELECT", 0
  BYTE "DRCAPTURE", 0, "DRSHIFT", 0, "DREXIT1", 0, "DRPAUSE", 0
  BYTE "DREXIT2", 0, "DRUPDATE", 0, "IRSELECT", 0, "IRCAPTURE", 0
  BYTE "IRSHIFT", 0, "IREXIT1", 0, "IRPAUSE", 0, "IREXIT2", 0
  BYTE "IRUPDATE", 0
  xsvf_pzTapState LONG @datstr_1 + 16, @datstr_1 + 22, @datstr_1 + 35
  LONG @datstr_1 + 44, @datstr_1 + 54, @datstr_1 + 62, @datstr_1 + 70
  LONG @datstr_1 + 78, @datstr_1 + 86, @datstr_1 + 95, @datstr_1 + 104
  LONG @datstr_1 + 114, @datstr_1 + 122, @datstr_1 + 130, @datstr_1 + 138
  LONG @datstr_1 + 146
{ DEBUG_MODE }

VAR
  LONG in
{ DEBUG_MODE }
{============================================================================
* Utility Functions
============================================================================}
{****************************************************************************
* Function:     xsvfPrintLenVal
* Description:  Print the lenval value in hex.
* Parameters:   plv     - ptr to lenval.
* Returns:      void.
****************************************************************************}

PUB xsvfPrintLenVal(plv) | i
  IF (plv)
    c.printf0(STRING("0x"))
    i := 0
    REPEAT WHILE (i < WORD[plv + len])
      c.printf1(STRING("%02x"),((BYTE[plv + val][i])))
      ++ i 
{ DEBUG_MODE }
{****************************************************************************
* Function:     xsvfInfoInit
* Description:  Initialize the xsvfInfo data.
* Parameters:   pXsvfInfo   - ptr to the XSVF info structure.
* Returns:      int         - 0 = success; otherwise error.
****************************************************************************}

PUB xsvfInfoInit(pXsvfInfo)
  XSVFDBG_PRINTF1(4, STRING("    sizeof( SXsvfInfo ) = %d bytes\n"), 17556)
  BYTE[pXsvfInfo + ucComplete] := 0
  BYTE[pXsvfInfo + ucCommand] := XCOMPLETE
  LONG[pXsvfInfo + lCommandCount] := 0
  LONG[pXsvfInfo + iErrorCode] := XSVF_ERROR_NONE
  BYTE[pXsvfInfo + ucMaxRepeat] := 0
  BYTE[pXsvfInfo + ucTapState] := XTAPSTATE_RESET
  BYTE[pXsvfInfo + ucEndIR] := XTAPSTATE_RUNTEST
  BYTE[pXsvfInfo + ucEndDR] := XTAPSTATE_RUNTEST
  LONG[pXsvfInfo + lShiftLengthBits] := 0
  WORD[pXsvfInfo + sShiftLengthBytes] := 0
  LONG[pXsvfInfo + lRunTestTime] := 0
  RETURN (0)
{****************************************************************************
* Function:     xsvfInfoCleanup
* Description:  Cleanup the xsvfInfo data.
* Parameters:   pXsvfInfo   - ptr to the XSVF info structure.
* Returns:      void.
****************************************************************************}

PUB xsvfInfoCleanup(pXsvfInfo)
{****************************************************************************
* Function:     xsvfGetAsNumBytes
* Description:  Calculate the number of bytes the given number of bits
*               consumes.
* Parameters:   lNumBits    - the number of bits.
* Returns:      short       - the number of bytes to store the number of bits.
****************************************************************************}

PUB xsvfGetAsNumBytes(lNumBits)
  RETURN (((lNumBits + 7) / 8))
{****************************************************************************
* Function:     xsvfTmsTransition
* Description:  Apply TMS and transition TAP controller by applying one TCK
*               cycle.
* Parameters:   sTms    - new TMS value.
* Returns:      void.
****************************************************************************}

PUB xsvfTmsTransition(sTms)
  setPort(TMS, sTms)
  setPort(TCK, 0)
  setPort(TCK, 1)
{****************************************************************************
* Function:     xsvfGotoTapState
* Description:  From the current TAP state, go to the named TAP state.
*               A target state of RESET ALWAYS causes TMS reset sequence.
*               All SVF standard stable state paths are supported.
*               All state transitions are supported except for the following
*               which cause an XSVF_ERROR_ILLEGALSTATE:
*                   - Target==DREXIT2;  Start!=DRPAUSE
*                   - Target==IREXIT2;  Start!=IRPAUSE
* Parameters:   pucTapState     - Current TAP state; returns final TAP state.
*               ucTargetState   - New target TAP state.
* Returns:      int             - 0 = success; otherwise error.
****************************************************************************}

PUB xsvfGotoTapState(pucTapState, ucTargetState) | i, iErrorCode_0, ucTapState_0
  iErrorCode_0 := XSVF_ERROR_NONE
  IF (ucTargetState == XTAPSTATE_RESET)
    { If RESET, always perform TMS reset sequence to reset/sync TAPs }
    xsvfTmsTransition(1)
    i := 0
    REPEAT WHILE (i < 5)
      setPort(TCK, 0)
      setPort(TCK, 1)
      ++ i 
    BYTE[pucTapState] := XTAPSTATE_RESET
    XSVFDBG_PRINTF(3, STRING("   TMS Reset Sequence -> Test-Logic-Reset\n"))
    ucTapState_0 := BYTE[pucTapState]
    XSVFDBG_PRINTF1(3, STRING("   TAP State = %s\n"), xsvf_pzTapState[ucTapState_0])
  ELSEIF ((ucTargetState <> BYTE[pucTapState]) AND(((ucTargetState == XTAPSTATE_EXIT2DR) AND(BYTE[pucTapState] <> XTAPSTATE_PAUSEDR)) OR((ucTargetState == XTAPSTATE_EXIT2IR) AND(BYTE[pucTapState] <> XTAPSTATE_PAUSEIR))))
    { Trap illegal TAP state path specification }
    iErrorCode_0 := XSVF_ERROR_ILLEGALSTATE
  else
    IF (ucTargetState == BYTE[pucTapState])
      { Already in target state.  Do nothing except when in DRPAUSE
                     or in IRPAUSE to comply with SVF standard }
      IF (ucTargetState == XTAPSTATE_PAUSEDR)
        xsvfTmsTransition(1)
        BYTE[pucTapState] := XTAPSTATE_EXIT2DR
        ucTapState_0 := BYTE[pucTapState]
        XSVFDBG_PRINTF1(3, STRING("   TAP State = %s\n"), xsvf_pzTapState[ucTapState_0])
      ELSEIF (ucTargetState == XTAPSTATE_PAUSEIR)
        xsvfTmsTransition(1)
        BYTE[pucTapState] := XTAPSTATE_EXIT2IR
        ucTapState_0 := BYTE[pucTapState]
        XSVFDBG_PRINTF1(3, STRING("   TAP State = %s\n"), xsvf_pzTapState[ucTapState_0])
    { Perform TAP state transitions to get to the target state }
    REPEAT WHILE (ucTargetState <> BYTE[pucTapState])
      CASE(BYTE[pucTapState])
        XTAPSTATE_RESET :
          xsvfTmsTransition(0)
          BYTE[pucTapState] := XTAPSTATE_RUNTEST
        XTAPSTATE_RUNTEST :
          xsvfTmsTransition(1)
          BYTE[pucTapState] := XTAPSTATE_SELECTDR
        XTAPSTATE_SELECTDR :
          IF (ucTargetState => XTAPSTATE_IRSTATES)
            xsvfTmsTransition(1)
            BYTE[pucTapState] := XTAPSTATE_SELECTIR
          else
            xsvfTmsTransition(0)
            BYTE[pucTapState] := XTAPSTATE_CAPTUREDR
        XTAPSTATE_CAPTUREDR :
          IF (ucTargetState == XTAPSTATE_SHIFTDR)
            xsvfTmsTransition(0)
            BYTE[pucTapState] := XTAPSTATE_SHIFTDR
          else
            xsvfTmsTransition(1)
            BYTE[pucTapState] := XTAPSTATE_EXIT1DR
        XTAPSTATE_SHIFTDR :
          xsvfTmsTransition(1)
          BYTE[pucTapState] := XTAPSTATE_EXIT1DR
        XTAPSTATE_EXIT1DR :
          IF (ucTargetState == XTAPSTATE_PAUSEDR)
            xsvfTmsTransition(0)
            BYTE[pucTapState] := XTAPSTATE_PAUSEDR
          else
            xsvfTmsTransition(1)
            BYTE[pucTapState] := XTAPSTATE_UPDATEDR
        XTAPSTATE_PAUSEDR :
          xsvfTmsTransition(1)
          BYTE[pucTapState] := XTAPSTATE_EXIT2DR
        XTAPSTATE_EXIT2DR :
          IF (ucTargetState == XTAPSTATE_SHIFTDR)
            xsvfTmsTransition(0)
            BYTE[pucTapState] := XTAPSTATE_SHIFTDR
          else
            xsvfTmsTransition(1)
            BYTE[pucTapState] := XTAPSTATE_UPDATEDR
        XTAPSTATE_UPDATEDR :
          IF (ucTargetState == XTAPSTATE_RUNTEST)
            xsvfTmsTransition(0)
            BYTE[pucTapState] := XTAPSTATE_RUNTEST
          else
            xsvfTmsTransition(1)
            BYTE[pucTapState] := XTAPSTATE_SELECTDR
        XTAPSTATE_SELECTIR :
          xsvfTmsTransition(0)
          BYTE[pucTapState] := XTAPSTATE_CAPTUREIR
        XTAPSTATE_CAPTUREIR :
          IF (ucTargetState == XTAPSTATE_SHIFTIR)
            xsvfTmsTransition(0)
            BYTE[pucTapState] := XTAPSTATE_SHIFTIR
          else
            xsvfTmsTransition(1)
            BYTE[pucTapState] := XTAPSTATE_EXIT1IR
        XTAPSTATE_SHIFTIR :
          xsvfTmsTransition(1)
          BYTE[pucTapState] := XTAPSTATE_EXIT1IR
        XTAPSTATE_EXIT1IR :
          IF (ucTargetState == XTAPSTATE_PAUSEIR)
            xsvfTmsTransition(0)
            BYTE[pucTapState] := XTAPSTATE_PAUSEIR
          else
            xsvfTmsTransition(1)
            BYTE[pucTapState] := XTAPSTATE_UPDATEIR
        XTAPSTATE_PAUSEIR :
          xsvfTmsTransition(1)
          BYTE[pucTapState] := XTAPSTATE_EXIT2IR
        XTAPSTATE_EXIT2IR :
          IF (ucTargetState == XTAPSTATE_SHIFTIR)
            xsvfTmsTransition(0)
            BYTE[pucTapState] := XTAPSTATE_SHIFTIR
          else
            xsvfTmsTransition(1)
            BYTE[pucTapState] := XTAPSTATE_UPDATEIR
        XTAPSTATE_UPDATEIR :
          IF (ucTargetState == XTAPSTATE_RUNTEST)
            xsvfTmsTransition(0)
            BYTE[pucTapState] := XTAPSTATE_RUNTEST
          else
            xsvfTmsTransition(1)
            BYTE[pucTapState] := XTAPSTATE_SELECTDR
        OTHER :
          iErrorCode_0 := XSVF_ERROR_ILLEGALSTATE
          BYTE[pucTapState] := ucTargetState { Exit while loop }
      ucTapState_0 := BYTE[pucTapState]
      XSVFDBG_PRINTF1(3, STRING("   TAP State = %s\n"), xsvf_pzTapState[ucTapState_0])
  RETURN (iErrorCode_0)
{****************************************************************************
* Function:     xsvfShiftOnly
* Description:  Assumes that starting TAP state is SHIFT-DR or SHIFT-IR.
*               Shift the given TDI data into the JTAG scan chain.
*               Optionally, save the TDO data shifted out of the scan chain.
*               Last shift cycle is special:  capture last TDO, set last TDI,
*               but does not pulse TCK.  Caller must pulse TCK and optionally
*               set TMS=1 to exit shift state.
* Parameters:   lNumBits        - number of bits to shift.
*               plvTdi          - ptr to lenval for TDI data.
*               plvTdoCaptured  - ptr to lenval for storing captured TDO data.
*               iExitShift      - 1=exit at end of shift; 0=stay in Shift-DR.
* Returns:      void.
****************************************************************************}

PUB xsvfShiftOnly(lNumBits, plvTdi, plvTdoCaptured, iExitShift) | pucTdi, pucTdo, ucTdiByte, ucTdoByte, ucTdoBit, i
  ' xxxxxxxxxxxxxxx
  ucTdiByte := 0
  ucTdoByte := 0
  ucTdoBit := 0
  { assert( ( ( lNumBits + 7 ) / 8 ) == plvTdi->len ); }
  { Initialize TDO storage len == TDI len }
  pucTdo := 0
  IF (plvTdoCaptured)
    WORD[plvTdoCaptured + len] := WORD[plvTdi + len]
    pucTdo :=(plvTdoCaptured + val) + WORD[plvTdi + len]
  { Shift LSB first.  val[N-1] == LSB.  val[0] == MSB. }
  pucTdi :=(plvTdi + val) + WORD[plvTdi + len]
  REPEAT WHILE (lNumBits)
    { Process on a byte-basis }
    --pucTdi
    ucTdiByte := BYTE[pucTdi]
    ucTdoByte := 0
    i := 0
    REPEAT WHILE ((lNumBits AND(i < 8)))
      --lNumBits
      IF (iExitShift AND NOT lNumBits)
        { Exit Shift-DR state }
        setPort(TMS, 1)
      { Set the new TDI value }
      setPort(TDI,(ucTdiByte & 1))
      ucTdiByte >>= 1
      { Set TCK low }
      setPort(TCK, 0)
      IF (pucTdo)
        { Save the TDO value }
        ucTdoBit := readTDOBit
        ucTdoByte |=(ucTdoBit << i)
      { Set TCK high }
      setPort(TCK, 1)
      ++ i 
    { Save the TDO byte value }
    IF (pucTdo)
      --pucTdo
      BYTE[pucTdo] := ucTdoByte
{****************************************************************************
* Function:     xsvfShift
* Description:  Goes to the given starting TAP state.
*               Calls xsvfShiftOnly to shift in the given TDI data and
*               optionally capture the TDO data.
*               Compares the TDO captured data against the TDO expected
*               data.
*               If a data mismatch occurs, then executes the exception
*               handling loop upto ucMaxRepeat times.
* Parameters:   pucTapState     - Ptr to current TAP state.
*               ucStartState    - Starting shift state: Shift-DR or Shift-IR.
*               lNumBits        - number of bits to shift.
*               plvTdi          - ptr to lenval for TDI data.
*               plvTdoCaptured  - ptr to lenval for storing TDO data.
*               plvTdoExpected  - ptr to expected TDO data.
*               plvTdoMask      - ptr to TDO mask.
*               ucEndState      - state in which to end the shift.
*               lRunTestTime    - amount of time to wait after the shift.
*               ucMaxRepeat     - Maximum number of retries on TDO mismatch.
* Returns:      int             - 0 = success; otherwise TDO mismatch.
* Notes:        XC9500XL-only Optimization:
*               Skip the waitTime() if plvTdoMask->val[0:plvTdoMask->len-1]
*               is NOT all zeros and sMatch==1.
****************************************************************************}

PUB xsvfShift(pucTapState, ucStartState, lNumBits, plvTdi, plvTdoCaptured, plvTdoExpected, plvTdoMask, ucEndState, lRunTestTime_0, ucMaxRepeat_0) | iErrorCode_0, iMismatch, ucRepeat, iExitShift, ucTapState_0
  iErrorCode_0 := XSVF_ERROR_NONE
  iMismatch := 0
  ucRepeat := 0
  iExitShift :=(ucStartState <> ucEndState)
  XSVFDBG_PRINTF1(3, STRING("   Shift Length = %ld\n"), lNumBits)
  XSVFDBG_PRINTF(4, STRING("    TDI          = "))
  XSVFDBG_PRINTLENVAL(4, plvTdi)
  XSVFDBG_PRINTF(4, STRING("\n"))
  XSVFDBG_PRINTF(4, STRING("    TDO Expected = "))
  XSVFDBG_PRINTLENVAL(4, plvTdoExpected)
  XSVFDBG_PRINTF(4, STRING("\n"))
  IF (NOT lNumBits)
    { Compatibility with XSVF2.00:  XSDR 0 = no shift, but wait in RTI }
    IF (lRunTestTime_0)
      { Wait for prespecified XRUNTEST time }
      xsvfGotoTapState(pucTapState, XTAPSTATE_RUNTEST)
      XSVFDBG_PRINTF1(3, STRING("   Wait = %ld usec\n"), lRunTestTime_0)
      waitTime(lRunTestTime_0)
  else
    REPEAT WHILE (1)
      { Goto Shift-DR or Shift-IR }
      xsvfGotoTapState(pucTapState, ucStartState)
      { Shift TDI and capture TDO }
      xsvfShiftOnly(lNumBits, plvTdi, plvTdoCaptured, iExitShift)
      IF (plvTdoExpected)
        { Compare TDO data to expected TDO data }
        iMismatch := NOT EqualLenVal(plvTdoExpected, plvTdoCaptured, plvTdoMask)
      IF (iExitShift)
        { Update TAP state:  Shift->Exit }
        BYTE[pucTapState][0]++
        ucTapState_0 := BYTE[pucTapState]
        XSVFDBG_PRINTF1(3, STRING("   TAP State = %s\n"), xsvf_pzTapState[ucTapState_0])
        IF (iMismatch AND lRunTestTime_0 AND(ucRepeat < ucMaxRepeat_0))
          XSVFDBG_PRINTF(4, STRING("    TDO Expected = "))
          XSVFDBG_PRINTLENVAL(4, plvTdoExpected)
          XSVFDBG_PRINTF(4, STRING("\n"))
          XSVFDBG_PRINTF(4, STRING("    TDO Captured = "))
          XSVFDBG_PRINTLENVAL(4, plvTdoCaptured)
          XSVFDBG_PRINTF(4, STRING("\n"))
          XSVFDBG_PRINTF(4, STRING("    TDO Mask     = "))
          XSVFDBG_PRINTLENVAL(4, plvTdoMask)
          XSVFDBG_PRINTF(4, STRING("\n"))
          XSVFDBG_PRINTF1(3, STRING("   Retry #%d\n"),(ucRepeat + 1))
          { Do exception handling retry - ShiftDR only }
          xsvfGotoTapState(pucTapState, XTAPSTATE_PAUSEDR)
          { Shift 1 extra bit }
          xsvfGotoTapState(pucTapState, XTAPSTATE_SHIFTDR)
          { Increment RUNTEST time by an additional 25% }
          lRunTestTime_0 +=(lRunTestTime_0 >> 2)
        else
          { Do normal exit from Shift-XR }
          xsvfGotoTapState(pucTapState, ucEndState)
        IF (lRunTestTime_0)
          { Wait for prespecified XRUNTEST time }
          xsvfGotoTapState(pucTapState, XTAPSTATE_RUNTEST)
          XSVFDBG_PRINTF1(3, STRING("   Wait = %ld usec\n"), lRunTestTime_0)
          waitTime(lRunTestTime_0)
      IF (NOT iMismatch)
        QUIT
      IF (ucRepeat++=> ucMaxRepeat_0)
        QUIT
  IF (iMismatch)
    XSVFDBG_PRINTF(1, STRING(" TDO Expected = "))
    XSVFDBG_PRINTLENVAL(1, plvTdoExpected)
    XSVFDBG_PRINTF(1, STRING("\n"))
    XSVFDBG_PRINTF(1, STRING(" TDO Captured = "))
    XSVFDBG_PRINTLENVAL(1, plvTdoCaptured)
    XSVFDBG_PRINTF(1, STRING("\n"))
    XSVFDBG_PRINTF(1, STRING(" TDO Mask     = "))
    XSVFDBG_PRINTLENVAL(1, plvTdoMask)
    XSVFDBG_PRINTF(1, STRING("\n"))
    IF (ucMaxRepeat_0 AND(ucRepeat > ucMaxRepeat_0))
      iErrorCode_0 := XSVF_ERROR_MAXRETRIES
    else
      iErrorCode_0 := XSVF_ERROR_TDOMISMATCH
  RETURN (iErrorCode_0)
{****************************************************************************
* Function:     xsvfBasicXSDRTDO
* Description:  Get the XSDRTDO parameters and execute the XSDRTDO command.
*               This is the common function for all XSDRTDO commands.
* Parameters:   pucTapState         - Current TAP state.
*               lShiftLengthBits    - number of bits to shift.
*               sShiftLengthBytes   - number of bytes to read.
*               plvTdi              - ptr to lenval for TDI data.
*               lvTdoCaptured       - ptr to lenval for storing TDO data.
*               iEndState           - state in which to end the shift.
*               lRunTestTime        - amount of time to wait after the shift.
*               ucMaxRepeat         - maximum xc9500/xl retries.
* Returns:      int                 - 0 = success; otherwise TDO mismatch.
****************************************************************************}

PUB xsvfBasicXSDRTDO(pucTapState, lShiftLengthBits_0, sShiftLengthBytes_0, plvTdi, plvTdoCaptured, plvTdoExpected, plvTdoMask, ucEndState, lRunTestTime_0, ucMaxRepeat_0)
  readVal(plvTdi, sShiftLengthBytes_0)
  IF (plvTdoExpected)
    readVal(plvTdoExpected, sShiftLengthBytes_0)
  RETURN (xsvfShift(pucTapState, XTAPSTATE_SHIFTDR, lShiftLengthBits_0, plvTdi, plvTdoCaptured, plvTdoExpected, plvTdoMask, ucEndState, lRunTestTime_0, ucMaxRepeat_0))
{****************************************************************************
* Function:     xsvfDoSDRMasking
* Description:  Update the data value with the next XSDRINC data and address.
* Example:      dataVal=0x01ff, nextData=0xab, addressMask=0x0100,
*               dataMask=0x00ff, should set dataVal to 0x02ab
* Parameters:   plvTdi          - The current TDI value.
*               plvNextData     - the next data value.
*               plvAddressMask  - the address mask.
*               plvDataMask     - the data mask.
* Returns:      void.
****************************************************************************}

PUB xsvfDoSDRMasking(plvTdi, plvNextData, plvAddressMask, plvDataMask) | i, ucTdi, ucTdiMask, ucDataMask, ucNextData, ucNextMask, sNextData
  'xxxxxxxxx
  ucTdi := 0
  ucTdiMask := 0
  ucDataMask := 0
  ucNextData := 0
  ucNextMask := 0
  sNextData := 0
  { add the address Mask to dataVal and return as a new dataVal }
  addVal(plvTdi, plvTdi, plvAddressMask)
  ucNextData := 0
  ucNextMask := 0
  sNextData := WORD[plvNextData + len]
  i := WORD[plvDataMask + len] - 1
  REPEAT WHILE (i => 0)
    { Go through data mask in reverse order looking for mask (1) bits }
    ucDataMask := BYTE[plvDataMask + val][i]
    IF (ucDataMask)
      { Retrieve the corresponding TDI byte value }
      ucTdi := BYTE[plvTdi + val][i]
      { For each bit in the data mask byte, look for 1's }
      ucTdiMask := 1
      REPEAT WHILE (ucDataMask)
        IF (ucDataMask & 1)
          IF (NOT ucNextMask)
            { Get the next data byte }
            ucNextData := BYTE[plvNextData + val][--sNextData]
            ucNextMask := 1
          { Set or clear the data bit according to the next data }
          IF (ucNextData & ucNextMask)
            ucTdi |= ucTdiMask { Set bit }
          else
            ucTdi &=(~ ucTdiMask) { Clear bit }
          { Update the next data }
          ucNextMask <<= 1
        ucTdiMask <<= 1
        ucDataMask >>= 1
      { Update the TDI value }
      BYTE[plvTdi + val][i] := ucTdi
    -- i 
{ XSVF_SUPPORT_COMPRESSION }
{============================================================================
* XSVF Command Functions (type = TXsvfDoCmdFuncPtr)
* These functions update pXsvfInfo->iErrorCode only on an error.
* Otherwise, the error code is left alone.
* The function returns the error code from the function.
============================================================================}
{****************************************************************************
* Function:     xsvfDoIllegalCmd
* Description:  Function place holder for illegal/unsupported commands.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoIllegalCmd(pXsvfInfo) | str, ucCommand_0
  ucCommand_0 := BYTE[pXsvfInfo + ucCommand]
  IF (ucCommand_0 < XLASTCMD)
    str := xsvf_pzCommandName[ucCommand_0]
  else
    str := STRING("Unknown")
  XSVFDBG_PRINTF2(0, STRING("ERROR:  Encountered unsupported command #%d (%s)\n"),((BYTE[pXsvfInfo + ucCommand])), str)
  LONG[pXsvfInfo + iErrorCode] := XSVF_ERROR_ILLEGALCMD
  RETURN (LONG[pXsvfInfo + iErrorCode])
{****************************************************************************
* Function:     xsvfDoXCOMPLETE
* Description:  XCOMPLETE (no parameters)
*               Update complete status for XSVF player.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXCOMPLETE(pXsvfInfo)
  BYTE[pXsvfInfo + ucComplete] := 1
  RETURN (XSVF_ERROR_NONE)
{****************************************************************************
* Function:     xsvfDoXTDOMASK
* Description:  XTDOMASK <lenVal.TdoMask[XSDRSIZE]>
*               Prespecify the TDO compare mask.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXTDOMASK(pXsvfInfo)
  readVal(@LONG[pXsvfInfo + lvTdoMask], WORD[pXsvfInfo + sShiftLengthBytes])
  XSVFDBG_PRINTF(4, STRING("    TDO Mask     = "))
  XSVFDBG_PRINTLENVAL(4, @LONG[pXsvfInfo + lvTdoMask])
  XSVFDBG_PRINTF(4, STRING("\n"))
  RETURN (XSVF_ERROR_NONE)
{****************************************************************************
* Function:     xsvfDoXSIR
* Description:  XSIR <(byte)shiftlen> <lenVal.TDI[shiftlen]>
*               Get the instruction and shift the instruction into the TAP.
*               If prespecified XRUNTEST!=0, goto RUNTEST and wait after
*               the shift for XRUNTEST usec.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXSIR(pXsvfInfo) | ucShiftIrBits, sShiftIrBytes, iErrorCode_0
  'xxxxxxxxxxxxx
  ucShiftIrBits := 0
  sShiftIrBytes := 0
  { Get the shift length and store }
  readByte(@ucShiftIrBits)
  sShiftIrBytes := xsvfGetAsNumBytes(ucShiftIrBits)
  XSVFDBG_PRINTF1(3, STRING("   XSIR length = %d\n"),(ucShiftIrBits))
  IF (sShiftIrBytes > MAX_LEN)
    iErrorCode_0 := XSVF_ERROR_DATAOVERFLOW
  else
    { Get and store instruction to shift in }
    readVal(@LONG[pXsvfInfo + lvTdi], xsvfGetAsNumBytes(ucShiftIrBits))
    { Shift the data }
    iErrorCode_0 := xsvfShift(@BYTE[pXsvfInfo + ucTapState], XTAPSTATE_SHIFTIR, ucShiftIrBits, @LONG[pXsvfInfo + lvTdi], 0, 0, 0, BYTE[pXsvfInfo + ucEndIR], LONG[pXsvfInfo + lRunTestTime], 0)
  IF (iErrorCode_0 <> XSVF_ERROR_NONE)
    LONG[pXsvfInfo + iErrorCode] := iErrorCode_0
  RETURN (iErrorCode_0)
{****************************************************************************
* Function:     xsvfDoXSIR2
* Description:  XSIR <(2-byte)shiftlen> <lenVal.TDI[shiftlen]>
*               Get the instruction and shift the instruction into the TAP.
*               If prespecified XRUNTEST!=0, goto RUNTEST and wait after
*               the shift for XRUNTEST usec.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXSIR2(pXsvfInfo) | lShiftIrBits, sShiftIrBytes, iErrorCode_0
  { Get the shift length and store }
  readVal(@LONG[pXsvfInfo + lvTdi], 2)
  lShiftIrBits := value(@LONG[pXsvfInfo + lvTdi])
  sShiftIrBytes := xsvfGetAsNumBytes(lShiftIrBits)
  XSVFDBG_PRINTF1(3, STRING("   XSIR2 length = %d\n"), lShiftIrBits)
  IF (sShiftIrBytes > MAX_LEN)
    iErrorCode_0 := XSVF_ERROR_DATAOVERFLOW
  else
    { Get and store instruction to shift in }
    readVal(@LONG[pXsvfInfo + lvTdi], xsvfGetAsNumBytes(lShiftIrBits))
    { Shift the data }
    iErrorCode_0 := xsvfShift(@BYTE[pXsvfInfo + ucTapState], XTAPSTATE_SHIFTIR, lShiftIrBits, @LONG[pXsvfInfo + lvTdi], 0, 0, 0, BYTE[pXsvfInfo + ucEndIR], LONG[pXsvfInfo + lRunTestTime], 0)
  IF (iErrorCode_0 <> XSVF_ERROR_NONE)
    LONG[pXsvfInfo + iErrorCode] := iErrorCode_0
  RETURN (iErrorCode_0)
{****************************************************************************
* Function:     xsvfDoXSDR
* Description:  XSDR <lenVal.TDI[XSDRSIZE]>
*               Shift the given TDI data into the JTAG scan chain.
*               Compare the captured TDO with the expected TDO from the
*               previous XSDRTDO command using the previously specified
*               XTDOMASK.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXSDR(pXsvfInfo) | iErrorCode_0
  readVal(@LONG[pXsvfInfo + lvTdi], WORD[pXsvfInfo + sShiftLengthBytes])
  { use TDOExpected from last XSDRTDO instruction }
  iErrorCode_0 := xsvfShift(@BYTE[pXsvfInfo + ucTapState], XTAPSTATE_SHIFTDR, LONG[pXsvfInfo + lShiftLengthBits], @LONG[pXsvfInfo + lvTdi], @LONG[pXsvfInfo + lvTdoCaptured], @LONG[pXsvfInfo + lvTdoExpected], @LONG[pXsvfInfo + lvTdoMask], BYTE[pXsvfInfo + ucEndDR], LONG[pXsvfInfo + lRunTestTime], BYTE[pXsvfInfo + ucMaxRepeat])
  IF (iErrorCode_0 <> XSVF_ERROR_NONE)
    LONG[pXsvfInfo + iErrorCode] := iErrorCode_0
  RETURN (iErrorCode_0)
{****************************************************************************
* Function:     xsvfDoXRUNTEST
* Description:  XRUNTEST <uint32>
*               Prespecify the XRUNTEST wait time for shift operations.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXRUNTEST(pXsvfInfo)
  readVal(@LONG[pXsvfInfo + lvTdi], 4)
  LONG[pXsvfInfo + lRunTestTime] := value(@LONG[pXsvfInfo + lvTdi])
  XSVFDBG_PRINTF1(3, STRING("   XRUNTEST = %ld\n"), LONG[pXsvfInfo + lRunTestTime])
  RETURN (XSVF_ERROR_NONE)
{****************************************************************************
* Function:     xsvfDoXREPEAT
* Description:  XREPEAT <byte>
*               Prespecify the maximum number of XC9500/XL retries.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXREPEAT(pXsvfInfo)
  readByte(@BYTE[pXsvfInfo + ucMaxRepeat])
  XSVFDBG_PRINTF1(3, STRING("   XREPEAT = %d\n"),((BYTE[pXsvfInfo + ucMaxRepeat])))
  RETURN (XSVF_ERROR_NONE)
{****************************************************************************
* Function:     xsvfDoXSDRSIZE
* Description:  XSDRSIZE <uint32>
*               Prespecify the XRUNTEST wait time for shift operations.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXSDRSIZE(pXsvfInfo) | iErrorCode_0
  iErrorCode_0 := XSVF_ERROR_NONE
  readVal(@LONG[pXsvfInfo + lvTdi], 4)
  LONG[pXsvfInfo + lShiftLengthBits] := value(@LONG[pXsvfInfo + lvTdi])
  WORD[pXsvfInfo + sShiftLengthBytes] := xsvfGetAsNumBytes(LONG[pXsvfInfo + lShiftLengthBits])
  XSVFDBG_PRINTF1(3, STRING("   XSDRSIZE = %ld\n"), LONG[pXsvfInfo + lShiftLengthBits])
  IF (WORD[pXsvfInfo + sShiftLengthBytes] > MAX_LEN)
    iErrorCode_0 := XSVF_ERROR_DATAOVERFLOW
    LONG[pXsvfInfo + iErrorCode] := iErrorCode_0
  RETURN (iErrorCode_0)
{****************************************************************************
* Function:     xsvfDoXSDRTDO
* Description:  XSDRTDO <lenVal.TDI[XSDRSIZE]> <lenVal.TDO[XSDRSIZE]>
*               Get the TDI and expected TDO values.  Then, shift.
*               Compare the expected TDO with the captured TDO using the
*               prespecified XTDOMASK.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXSDRTDO(pXsvfInfo) | iErrorCode_0
  iErrorCode_0 := xsvfBasicXSDRTDO(@BYTE[pXsvfInfo + ucTapState], LONG[pXsvfInfo + lShiftLengthBits], WORD[pXsvfInfo + sShiftLengthBytes], @LONG[pXsvfInfo + lvTdi], @LONG[pXsvfInfo + lvTdoCaptured], @LONG[pXsvfInfo + lvTdoExpected], @LONG[pXsvfInfo + lvTdoMask], BYTE[pXsvfInfo + ucEndDR], LONG[pXsvfInfo + lRunTestTime], BYTE[pXsvfInfo + ucMaxRepeat])
  IF (iErrorCode_0 <> XSVF_ERROR_NONE)
    LONG[pXsvfInfo + iErrorCode] := iErrorCode_0
  RETURN (iErrorCode_0)
{****************************************************************************
* Function:     xsvfDoXSETSDRMASKS
* Description:  XSETSDRMASKS <lenVal.AddressMask[XSDRSIZE]>
*                            <lenVal.DataMask[XSDRSIZE]>
*               Get the prespecified address and data mask for the XSDRINC
*               command.
*               Used for xc9500/xl compressed XSVF data.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXSETSDRMASKS(pXsvfInfo)
  { read the addressMask }
  readVal(@LONG[pXsvfInfo + lvAddressMask], WORD[pXsvfInfo + sShiftLengthBytes])
  { read the dataMask    }
  readVal(@LONG[pXsvfInfo + lvDataMask], WORD[pXsvfInfo + sShiftLengthBytes])
  XSVFDBG_PRINTF(4, STRING("    Address Mask = "))
  XSVFDBG_PRINTLENVAL(4, @LONG[pXsvfInfo + lvAddressMask])
  XSVFDBG_PRINTF(4, STRING("\n"))
  XSVFDBG_PRINTF(4, STRING("    Data Mask    = "))
  XSVFDBG_PRINTLENVAL(4, @LONG[pXsvfInfo + lvDataMask])
  XSVFDBG_PRINTF(4, STRING("\n"))
  RETURN (XSVF_ERROR_NONE)
{ XSVF_SUPPORT_COMPRESSION }
{****************************************************************************
* Function:     xsvfDoXSDRINC
* Description:  XSDRINC <lenVal.firstTDI[XSDRSIZE]> <byte(numTimes)>
*                       <lenVal.data[XSETSDRMASKS.dataMask.len]> ...
*               Get the XSDRINC parameters and execute the XSDRINC command.
*               XSDRINC starts by loading the first TDI shift value.
*               Then, for numTimes, XSDRINC gets the next piece of data,
*               replaces the bits from the starting TDI as defined by the
*               XSETSDRMASKS.dataMask, adds the address mask from
*               XSETSDRMASKS.addressMask, shifts the new TDI value,
*               and compares the TDO to the expected TDO from the previous
*               XSDRTDO command using the XTDOMASK.
*               Used for xc9500/xl compressed XSVF data.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXSDRINC(pXsvfInfo) | iErrorCode_0, iDataMaskLen, ucDataMask, ucNumTimes, i
  ' xxxxxx
  ucDataMask := 0
  ucNumTimes := 0
  i := 0
  readVal(@LONG[pXsvfInfo + lvTdi], WORD[pXsvfInfo + sShiftLengthBytes])
  iErrorCode_0 := xsvfShift(@BYTE[pXsvfInfo + ucTapState], XTAPSTATE_SHIFTDR, LONG[pXsvfInfo + lShiftLengthBits], @LONG[pXsvfInfo + lvTdi], @LONG[pXsvfInfo + lvTdoCaptured], @LONG[pXsvfInfo + lvTdoExpected], @LONG[pXsvfInfo + lvTdoMask], BYTE[pXsvfInfo + ucEndDR], LONG[pXsvfInfo + lRunTestTime], BYTE[pXsvfInfo + ucMaxRepeat])
  IF (NOT iErrorCode_0)
    { Calculate number of data mask bits }
    iDataMaskLen := 0
    i := 0
    REPEAT WHILE (i < WORD[pXsvfInfo + lvDataMask + len])
      ucDataMask := BYTE[pXsvfInfo + lvDataMask + val][i]
      REPEAT WHILE (ucDataMask)
        iDataMaskLen +=(ucDataMask & 1)
        ucDataMask >>= 1
      ++ i 
    { Get the number of data pieces, i.e. number of times to shift }
    readByte(@ucNumTimes)
    { For numTimes, get data, fix TDI, and shift }
    i := 0
    REPEAT WHILE (NOT iErrorCode_0 AND(i < ucNumTimes))
      readVal(@LONG[pXsvfInfo + lvNextData], xsvfGetAsNumBytes(iDataMaskLen))
      xsvfDoSDRMasking(@LONG[pXsvfInfo + lvTdi], @LONG[pXsvfInfo + lvNextData], @LONG[pXsvfInfo + lvAddressMask], @LONG[pXsvfInfo + lvDataMask])
      iErrorCode_0 := xsvfShift(@BYTE[pXsvfInfo + ucTapState], XTAPSTATE_SHIFTDR, LONG[pXsvfInfo + lShiftLengthBits], @LONG[pXsvfInfo + lvTdi], @LONG[pXsvfInfo + lvTdoCaptured], @LONG[pXsvfInfo + lvTdoExpected], @LONG[pXsvfInfo + lvTdoMask], BYTE[pXsvfInfo + ucEndDR], LONG[pXsvfInfo + lRunTestTime], BYTE[pXsvfInfo + ucMaxRepeat])
      ++ i 
  IF (iErrorCode_0 <> XSVF_ERROR_NONE)
    LONG[pXsvfInfo + iErrorCode] := iErrorCode_0
  RETURN (iErrorCode_0)
{ XSVF_SUPPORT_COMPRESSION }
{****************************************************************************
* Function:     xsvfDoXSDRBCE
* Description:  XSDRB/XSDRC/XSDRE <lenVal.TDI[XSDRSIZE]>
*               If not already in SHIFTDR, goto SHIFTDR.
*               Shift the given TDI data into the JTAG scan chain.
*               Ignore TDO.
*               If cmd==XSDRE, then goto ENDDR.  Otherwise, stay in ShiftDR.
*               XSDRB, XSDRC, and XSDRE are the same implementation.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXSDRBCE(pXsvfInfo) | ucEndDR_0, iErrorCode_0
  'xxxx
  ucEndDR_0 := 0
  IF (BYTE[pXsvfInfo + ucCommand] == XSDRE)
    ucEndDR_0 := BYTE[pXsvfInfo + ucEndDR]
  else
    ucEndDR_0 := XTAPSTATE_SHIFTDR
  iErrorCode_0 := xsvfBasicXSDRTDO(@BYTE[pXsvfInfo + ucTapState], LONG[pXsvfInfo + lShiftLengthBits], WORD[pXsvfInfo + sShiftLengthBytes], @LONG[pXsvfInfo + lvTdi], 0, 0, 0, ucEndDR_0, 0, 0)
  IF (iErrorCode_0 <> XSVF_ERROR_NONE)
    LONG[pXsvfInfo + iErrorCode] := iErrorCode_0
  RETURN (iErrorCode_0)
{****************************************************************************
* Function:     xsvfDoXSDRTDOBCE
* Description:  XSDRB/XSDRC/XSDRE <lenVal.TDI[XSDRSIZE]> <lenVal.TDO[XSDRSIZE]>
*               If not already in SHIFTDR, goto SHIFTDR.
*               Shift the given TDI data into the JTAG scan chain.
*               Compare TDO, but do NOT use XTDOMASK.
*               If cmd==XSDRTDOE, then goto ENDDR.  Otherwise, stay in ShiftDR.
*               XSDRTDOB, XSDRTDOC, and XSDRTDOE are the same implementation.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXSDRTDOBCE(pXsvfInfo) | ucEndDR_0, iErrorCode_0
  'xxxx
  ucEndDR_0 := 0
  IF (BYTE[pXsvfInfo + ucCommand] == XSDRTDOE)
    ucEndDR_0 := BYTE[pXsvfInfo + ucEndDR]
  else
    ucEndDR_0 := XTAPSTATE_SHIFTDR
  iErrorCode_0 := xsvfBasicXSDRTDO(@BYTE[pXsvfInfo + ucTapState], LONG[pXsvfInfo + lShiftLengthBits], WORD[pXsvfInfo + sShiftLengthBytes], @LONG[pXsvfInfo + lvTdi], @LONG[pXsvfInfo + lvTdoCaptured], @LONG[pXsvfInfo + lvTdoExpected], 0, ucEndDR_0, 0, 0)
  IF (iErrorCode_0 <> XSVF_ERROR_NONE)
    LONG[pXsvfInfo + iErrorCode] := iErrorCode_0
  RETURN (iErrorCode_0)
{****************************************************************************
* Function:     xsvfDoXSTATE
* Description:  XSTATE <byte>
*               <byte> == XTAPSTATE;
*               Get the state parameter and transition the TAP to that state.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXSTATE(pXsvfInfo) | ucNextState, iErrorCode_0
  'xxxx
  ucNextState := 0
  readByte(@ucNextState)
  iErrorCode_0 := xsvfGotoTapState(@BYTE[pXsvfInfo + ucTapState], ucNextState)
  IF (iErrorCode_0 <> XSVF_ERROR_NONE)
    LONG[pXsvfInfo + iErrorCode] := iErrorCode_0
  RETURN (iErrorCode_0)
{****************************************************************************
* Function:     xsvfDoXENDXR
* Description:  XENDIR/XENDDR <byte>
*               <byte>:  0 = RUNTEST;  1 = PAUSE.
*               Get the prespecified XENDIR or XENDDR.
*               Both XENDIR and XENDDR use the same implementation.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXENDXR(pXsvfInfo) | iErrorCode_0, ucEndState
  'xxxxx
  ucEndState := 0
  iErrorCode_0 := XSVF_ERROR_NONE
  readByte(@ucEndState)
  IF ((ucEndState <> XENDXR_RUNTEST) AND(ucEndState <> XENDXR_PAUSE))
    iErrorCode_0 := XSVF_ERROR_ILLEGALSTATE
  else
    IF (BYTE[pXsvfInfo + ucCommand] == XENDIR)
      IF (ucEndState == XENDXR_RUNTEST)
        BYTE[pXsvfInfo + ucEndIR] := XTAPSTATE_RUNTEST
      else
        BYTE[pXsvfInfo + ucEndIR] := XTAPSTATE_PAUSEIR
      XSVFDBG_PRINTF1(3, STRING("   ENDIR State = %s\n"), xsvf_pzTapState[pXsvfInfo->ucEndIR])
    else { XENDDR }
      IF (ucEndState == XENDXR_RUNTEST)
        BYTE[pXsvfInfo + ucEndDR] := XTAPSTATE_RUNTEST
      else
        BYTE[pXsvfInfo + ucEndDR] := XTAPSTATE_PAUSEDR
      XSVFDBG_PRINTF1(3, STRING("   ENDDR State = %s\n"), xsvf_pzTapState[pXsvfInfo->ucEndDR])
  IF (iErrorCode_0 <> XSVF_ERROR_NONE)
    LONG[pXsvfInfo + iErrorCode] := iErrorCode_0
  RETURN (iErrorCode_0)
{****************************************************************************
* Function:     xsvfDoXCOMMENT
* Description:  XCOMMENT <text string ending in \0>
*               <text string ending in \0> == text comment;
*               Arbitrary comment embedded in the XSVF.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXCOMMENT(pXsvfInfo) | ucText
  'xxxxx
  ucText := 0
  { Use the comment for debugging }
  { Otherwise, read through the comment to the end '\0' and ignore }
  IF (xsvf_iDebugLevel > 0)
    c.putchar(" ")
  ucText := 1
  REPEAT WHILE (ucText)
    readByte(@ucText)
    IF (xsvf_iDebugLevel > 0)
      IF (ucText)
        c.putchar(ucText)
      else
        c.putchar(13)
  LONG[pXsvfInfo + iErrorCode] := XSVF_ERROR_NONE
  RETURN (LONG[pXsvfInfo + iErrorCode])
{****************************************************************************
* Function:     xsvfDoXWAIT
* Description:  XWAIT <wait_state> <end_state> <wait_time>
*               If not already in <wait_state>, then go to <wait_state>.
*               Wait in <wait_state> for <wait_time> microseconds.
*               Finally, if not already in <end_state>, then goto <end_state>.
* Parameters:   pXsvfInfo   - XSVF information pointer.
* Returns:      int         - 0 = success;  non-zero = error.
****************************************************************************}

PUB xsvfDoXWAIT(pXsvfInfo) | ucWaitState, ucEndState, lWaitTime
  'xxxx
  ucWaitState := 0
  ucEndState := 0
  { Get Parameters }
  { <wait_state> }
  readVal(@LONG[pXsvfInfo + lvTdi], 1)
  ucWaitState := BYTE[pXsvfInfo + lvTdi + val][0]
  { <end_state> }
  readVal(@LONG[pXsvfInfo + lvTdi], 1)
  ucEndState := BYTE[pXsvfInfo + lvTdi + val][0]
  { <wait_time> }
  readVal(@LONG[pXsvfInfo + lvTdi], 4)
  lWaitTime := value(@LONG[pXsvfInfo + lvTdi])
  XSVFDBG_PRINTF2(3, STRING("   XWAIT:  state = %s; time = %ld\n"), xsvf_pzTapState[ucWaitState], lWaitTime)
  { If not already in <wait_state>, go to <wait_state> }
  IF (BYTE[pXsvfInfo + ucTapState] <> ucWaitState)
    xsvfGotoTapState(@BYTE[pXsvfInfo + ucTapState], ucWaitState)
  { Wait for <wait_time> microseconds }
  waitTime(lWaitTime)
  { If not already in <end_state>, go to <end_state> }
  IF (BYTE[pXsvfInfo + ucTapState] <> ucEndState)
    xsvfGotoTapState(@BYTE[pXsvfInfo + ucTapState], ucEndState)
  RETURN (XSVF_ERROR_NONE)
{============================================================================
* Execution Control Functions
============================================================================}
{****************************************************************************
* Function:     xsvfInitialize
* Description:  Initialize the xsvf player.
*               Call this before running the player to initialize the data
*               in the SXsvfInfo struct.
*               xsvfCleanup is called to clean up the data in SXsvfInfo
*               after the XSVF is played.
* Parameters:   pXsvfInfo   - ptr to the XSVF information.
* Returns:      int - 0 = success; otherwise error.
****************************************************************************}

PUB xsvfInitialize(pXsvfInfo)
  { Initialize values }
  LONG[pXsvfInfo + iErrorCode] := xsvfInfoInit(pXsvfInfo)
  IF (NOT LONG[pXsvfInfo + iErrorCode])
    { Initialize the TAPs }
    LONG[pXsvfInfo + iErrorCode] := xsvfGotoTapState(@BYTE[pXsvfInfo + ucTapState], XTAPSTATE_RESET)
  RETURN (LONG[pXsvfInfo + iErrorCode])
{****************************************************************************
* Function:     xsvfRun
* Description:  Run the xsvf player for a single command and return.
*               First, call xsvfInitialize.
*               Then, repeatedly call this function until an error is detected
*               or until the pXsvfInfo->ucComplete variable is non-zero.
*               Finally, call xsvfCleanup to cleanup any remnants.
* Parameters:   pXsvfInfo   - ptr to the XSVF information.
* Returns:      int         - 0 = success; otherwise error.
****************************************************************************}

PUB xsvfRun(pXsvfInfo) | ucCommand_0
  ucCommand_0 := BYTE[pXsvfInfo + ucCommand]
  { Process the XSVF commands }
  IF ((NOT LONG[pXsvfInfo + iErrorCode]) AND(NOT BYTE[pXsvfInfo + ucComplete]))
    { read 1 byte for the instruction }
    readByte(@BYTE[pXsvfInfo + ucCommand])
    LONG[pXsvfInfo + lCommandCount]++
    IF (BYTE[pXsvfInfo + ucCommand] < XLASTCMD)
      { Execute the command.  Func sets error code. }
      XSVFDBG_PRINTF1(2, STRING("  %s\n"), xsvf_pzCommandName[ucCommand_0])
      { If your compiler cannot take this form,
                     then convert to a switch statement }
      xsvf_DoCmd(BYTE[pXsvfInfo + ucCommand], pXsvfInfo)
    else
      { Illegal command value.  Func sets error code. }
      xsvfDoIllegalCmd(pXsvfInfo)
  RETURN (LONG[pXsvfInfo + iErrorCode])
{****************************************************************************
* Function:     xsvfCleanup
* Description:  cleanup remnants of the xsvf player.
* Parameters:   pXsvfInfo   - ptr to the XSVF information.
* Returns:      void.
****************************************************************************}

PUB xsvfCleanup(pXsvfInfo)
  xsvfInfoCleanup(pXsvfInfo)
{============================================================================
* xsvfExecute() - The primary entry point to the XSVF player
============================================================================}
{****************************************************************************
* Function:     xsvfExecute
* Description:  Process, interpret, and apply the XSVF commands.
*               See port.c:readByte for source of XSVF data.
* Parameters:   none.
* Returns:      int - Legacy result values:  1 == success;  0 == failed.
****************************************************************************}

VAR
  LONG xsvfInfo[4389]

PUB xsvfExecute | iErrorCode_0
  xsvfInitialize(@xsvfInfo)
  REPEAT WHILE (NOT LONG[@xsvfInfo + iErrorCode] AND(NOT BYTE[@xsvfInfo + ucComplete]))
    xsvfRun(@xsvfInfo)
  IF (LONG[@xsvfInfo + iErrorCode])
    iErrorCode_0 := LONG[@xsvfInfo + iErrorCode]
    IF (iErrorCode_0 < XSVF_ERROR_LAST)
      XSVFDBG_PRINTF1(0, STRING("%s\n"), xsvf_pzErrorName[iErrorCode_0])
    else
      XSVFDBG_PRINTF1(0, STRING("%s\n"), xsvf_pzErrorName[XSVF_ERROR_UNKNOWN])
    XSVFDBG_PRINTF2(0, STRING("ERROR at or near XSVF command #%ld.  See line #%ld in the XSVF ASCII file.\n"), LONG[@xsvfInfo + lCommandCount], LONG[@xsvfInfo + lCommandCount])
  else
    XSVFDBG_PRINTF(0, STRING("SUCCESS - Completed XSVF execution.\n"))
  xsvfCleanup(@xsvfInfo)
  RETURN (XSVF_ERRORCODE(LONG[@xsvfInfo + iErrorCode]))
{============================================================================
* main
============================================================================}
{****************************************************************************
* Function:     main
* Description:  main function.
*               Specified here for creating stand-alone debug executable.
*               Embedded users should call xsvfExecute() directly.
* Parameters:   iArgc    - number of command-line arguments.
*               ppzArgv  - array of ptrs to strings (command-line arguments).
* Returns:      int      - Legacy return value:  1 = success; 0 = error.
****************************************************************************}

PUB toupper(val_0)
  IF (val_0 => "a" AND val_0 =< "z")
    val_0 -= "a" - "A"
  RETURN val_0

PUB _stricmp(str1, str2) | val1, val2
  val1 := toupper(BYTE[str1++])
  val2 := toupper(BYTE[str2++])
  REPEAT WHILE (val1 AND val2 AND val1 == val2)
    val1 := toupper(BYTE[str1++])
    val2 := toupper(BYTE[str2++])
  RETURN (val1 <> val2)

PUB atoi(str) | i, sign_0
  i := 0
  sign_0 := 1
  REPEAT WHILE (BYTE[str] == " ")
    str++
  IF (BYTE[str] == "-")
    sign_0 := - 1
    str++
  ELSEIF (BYTE[str] == "+")
    str++
  REPEAT WHILE (BYTE[str] => "0" AND BYTE[str] =< "9")
    i :=(i * 10) +(BYTE[str++]) - "0"
  RETURN i * sign_0

PUB fopen(fname, attrib)
  RETURN 1

PUB fclose(stream)
  RETURN 1

CON
  CLOCKS_PER_MSEC = 80000

PUB clock
  RETURN cnt

DAT
  iArgc LONG 4
  datstr_2 BYTE "micro", 0, "-v", 0, "4", 0, "example.xsvf", 0
  ppzArgv LONG @datstr_2 + 16, @datstr_2 + 22, @datstr_2 + 25, @datstr_2 + 27

PUB main | iErrorCode_0, pzXsvfFileName, i, startClock, endClock
  i := 10
  REPEAT WHILE (i => 0)
    c.printf1(STRING("i = %d\n"), i)
    waitcnt(clkfreq + cnt)
    i -- 
  iErrorCode_0 := XSVF_ERRORCODE(XSVF_ERROR_NONE)
  pzXsvfFileName := 0
  c.printf1(STRING("XSVF Player v%s, Xilinx, Inc.\n"), STRING("5.01"))
  i := 1
  REPEAT WHILE (i < iArgc)
    IF (NOT _stricmp(ppzArgv[i], STRING("-v")))
      ++i
      IF (i => iArgc)
        c.printf0(STRING("ERROR:  missing <level> parameter for -v option.\n"))
      else
        xsvf_iDebugLevel := atoi(ppzArgv[i])
        c.printf1(STRING("Verbose level = %d\n"), xsvf_iDebugLevel)
    else
      pzXsvfFileName := ppzArgv[i]
      c.printf1(STRING("XSVF file = %s\n"), pzXsvfFileName)
    ++ i 
  IF (NOT pzXsvfFileName)
    c.printf0(STRING("USAGE:  playxsvf [-v level] filename.xsvf\n"))
    c.printf0(STRING("where:  -v level      = verbose, level = 0-4 (default=0)\n"))
    c.printf0(STRING("        filename.xsvf = the XSVF file to execute.\n"))
  else
    { read from the XSVF file instead of a real prom }
    in := fopen(pzXsvfFileName, STRING("rb"))
    IF (NOT in)
      c.printf1(STRING("ERROR:  Cannot open file %s\n"), pzXsvfFileName)
      iErrorCode_0 := XSVF_ERRORCODE(XSVF_ERROR_UNKNOWN)
    else
      { Initialize the I/O.  SetPort initializes I/O on first call }
      setPort(TMS, 1)
      { Execute the XSVF in the file }
      startClock := clock
      iErrorCode_0 := xsvfExecute
      endClock := clock
      fclose(in)
      c.printf1(STRING("Execution Time = %d msecs\n"),(endClock - startClock) / CLOCKS_PER_MSEC)
  RETURN (iErrorCode_0)
{ XSVF_MAIN }
'
' Include lenval.c
'
{*****************************************************}
{ file: lenval.c                                      }
{ abstract:  This file contains routines for using    }
{            the lenVal data structure.               }
{*****************************************************}
{*****************************************************}
{ file: lenval.h                                      }
{ abstract:  This file contains a description of the  }
{            data structure "lenval".                 }
{*****************************************************}
{*****************************************************}
{ file: ports.h                                       }
{ abstract:  This file contains extern declarations   }
{            for providing stimulus to the JTAG ports.}
{*****************************************************}
{****************************************************************************
* Function:     value
* Description:  Extract the long value from the lenval array.
* Parameters:   plvValue    - ptr to lenval.
* Returns:      long        - the extracted value.
****************************************************************************}

PUB value(plvValue) | lValue, sIndex
  lValue := 0
  sIndex := 0
  REPEAT WHILE (sIndex < WORD[plvValue + len])
    lValue <<= 8 { shift the accumulated result }
    lValue |= BYTE[plvValue + val][sIndex] { get the last byte first }
    ++ sIndex 
  RETURN (lValue)
{****************************************************************************
* Function:     initLenVal
* Description:  Initialize the lenval array with the given value.
*               Assumes lValue is less than 256.
* Parameters:   plv         - ptr to lenval.
*               lValue      - the value to set.
* Returns:      void.
****************************************************************************}

PUB initLenVal(plv, lValue)
  WORD[plv + len] := 1
  BYTE[plv + val][0] := lValue
{****************************************************************************
* Function:     EqualLenVal
* Description:  Compare two lenval arrays with an optional mask.
* Parameters:   plvTdoExpected  - ptr to lenval #1.
*               plvTdoCaptured  - ptr to lenval #2.
*               plvTdoMask      - optional ptr to mask (=0 if no mask).
* Returns:      short   - 0 = mismatch; 1 = equal.
****************************************************************************}

PUB EqualLenVal(plvTdoExpected, plvTdoCaptured, plvTdoMask) | sEqual, sIndex, ucByteVal1, ucByteVal2, ucByteMask
  sEqual := 1
  sIndex := WORD[plvTdoExpected + len]
  REPEAT WHILE (sEqual AND sIndex--)
    ucByteVal1 := BYTE[plvTdoExpected + val][sIndex]
    ucByteVal2 := BYTE[plvTdoCaptured + val][sIndex]
    IF (plvTdoMask)
      ucByteMask := BYTE[plvTdoMask + val][sIndex]
      ucByteVal1 &= ucByteMask
      ucByteVal2 &= ucByteMask
    IF (ucByteVal1 <> ucByteVal2)
      sEqual := 0
  RETURN (sEqual)
{****************************************************************************
* Function:     RetBit
* Description:  return the (byte, bit) of lv (reading from left to right).
* Parameters:   plv     - ptr to lenval.
*               iByte   - the byte to get the bit from.
*               iBit    - the bit number (0=msb)
* Returns:      short   - the bit value.
****************************************************************************}

PUB RetBit(plv, iByte, iBit)
  { assert( ( iByte >= 0 ) && ( iByte < plv->len ) ); }
  { assert( ( iBit >= 0 ) && ( iBit < 8 ) ); }
  RETURN (((BYTE[plv + val][iByte] >>(7 - iBit)) & $1))
{****************************************************************************
* Function:     SetBit
* Description:  set the (byte, bit) of lv equal to val
* Example:      SetBit("00000000",byte, 1) equals "01000000".
* Parameters:   plv     - ptr to lenval.
*               iByte   - the byte to get the bit from.
*               iBit    - the bit number (0=msb).
*               sVal    - the bit value to set.
* Returns:      void.
****************************************************************************}

PUB SetBit(plv, iByte, iBit, sVal) | ucByteVal, ucBitMask
  ucBitMask :=(1 <<(7 - iBit))
  ucByteVal :=(BYTE[plv + val][iByte] &(~ ucBitMask))
  IF (sVal)
    ucByteVal |= ucBitMask
  BYTE[plv + val][iByte] := ucByteVal
{****************************************************************************
* Function:     AddVal
* Description:  add val1 to val2 and store in resVal;
*               assumes val1 and val2  are of equal length.
* Parameters:   plvResVal   - ptr to result.
*               plvVal1     - ptr of addendum.
*               plvVal2     - ptr of addendum.
* Returns:      void.
****************************************************************************}

PUB addVal(plvResVal, plvVal1, plvVal2) | ucCarry, usSum, usVal1, usVal2, sIndex
  WORD[plvResVal + len] := WORD[plvVal1 + len] { set up length of result }
  { start at least significant bit and add bytes    }
  ucCarry := 0
  sIndex := WORD[plvVal1 + len]
  REPEAT WHILE (sIndex--)
    usVal1 := BYTE[plvVal1 + val][sIndex] { i'th byte of val1 }
    usVal2 := BYTE[plvVal2 + val][sIndex] { i'th byte of val2 }
    { add the two bytes plus carry from previous addition }
    usSum :=(usVal1 + usVal2 + ucCarry)
    { set up carry for next byte }
    IF (usSum > 255)
      ucCarry := 1
    else
      ucCarry := 0
    { set the i'th byte of the result }
    BYTE[plvResVal + val][sIndex] := usSum
{****************************************************************************
* Function:     readVal
* Description:  read from XSVF numBytes bytes of data into x.
* Parameters:   plv         - ptr to lenval in which to put the bytes read.
*               sNumBytes   - the number of bytes to read.
* Returns:      void.
****************************************************************************}

PUB readVal(plv, sNumBytes) | pucVal
  WORD[plv + len] := sNumBytes { set the length of the lenVal        }
  pucVal :=(plv + val)
  REPEAT WHILE (sNumBytes)
    { read a byte of data into the lenVal }
    readByte(pucVal++)
    -- sNumBytes 
'
' Include ports.c
'
{*****************************************************}
{ file: ports.c                                       }
{ abstract:  This file contains the routines to       }
{            output values on the JTAG ports, to read }
{            the TDO bit, and to read a byte of data  }
{            from the prom                            }
{ Revisions:                                          }
{ 12/01/2008:  Same code as before (original v5.01).  }
{              Updated comments to clarify instructions.}
{              Add print in setPort for xapp058_example.exe.}
{*****************************************************}
{*****************************************************}
{ file: ports.h                                       }
{ abstract:  This file contains extern declarations   }
{            for providing stimulus to the JTAG ports.}
{*****************************************************}
{#include "prgispx.h"}

DAT
  g_iTCK LONG 0
{ For xapp058_example .exe }
  g_iTMS LONG 0
{ For xapp058_example .exe }
  g_iTDI LONG 0
{ For xapp058_example .exe }
{BYTE *xsvf_data=0;}
{ setPort:  Implement to set the named JTAG signal (p) to the new value (v).}
{ if in debugging mode, then just set the variables }

PUB setPort(p, val_0)
  { Printing code for the xapp058_example.exe.  You must set the specified
         JTAG signal (p) to the new value (v).  See the above, old Win95 code
         as an implementation example. }
  IF (p == TMS)
    g_iTMS := val_0
  IF (p == TDI)
    g_iTDI := val_0
  IF (p == TCK)
    g_iTCK := val_0
    c.printf3(STRING("TCK = %d;  TMS = %d;  TDI = %d\n"), g_iTCK, g_iTMS, g_iTDI)
{ toggle tck LH.  No need to modify this code.  It is output via setPort. }

PUB pulseClock
  setPort(TCK, 0) { set the TCK port to low  }
  setPort(TCK, 1) { set the TCK port to high }
{ readByte:  Implement to source the next byte from your XSVF file location }
{ read in a byte of data from the prom }

DAT
  example_index LONG 0
  example_xsvf BYTE 7, 0, 18, 0, 2, 8, 53, 23, 1, 1, 0, 0, 0, 10, 8, 0, 0, 0, 8, 1, 255, 9, 0, 121, 0

PUB readByte(data)
  BYTE[data] := example_xsvf[example_index]
  IF (example_index < 24)
    example_index++
{ readTDOBit:  Implement to return the current value of the JTAG TDO signal.}
{ read the TDO bit from port }

PUB readTDOBit
  { You must return the current value of the JTAG TDO signal. }
  RETURN (0)
{ waitTime:  Implement as follows: }
{ REQUIRED:  This function must consume/wait at least the specified number  }
{            of microsec, interpreting microsec as a number of microseconds.}
{ REQUIRED FOR SPARTAN/VIRTEX FPGAs and indirect flash programming:         }
{            This function must pulse TCK for at least microsec times,      }
{            interpreting microsec as an integer value.                     }
{ RECOMMENDED IMPLEMENTATION:  Pulse TCK at least microsec times AND        }
{                              continue pulsing TCK until the microsec wait }
{                              requirement is also satisfied.               }

PUB waitTime(microsec) | tckCyclesPerMicrosec, tckCycles, i
  tckCyclesPerMicrosec := 1
  tckCycles := microsec * tckCyclesPerMicrosec
  { This implementation is highly recommended!!! }
  { This implementation requires you to tune the tckCyclesPerMicrosec 
         variable (above) to match the performance of your embedded system
         in order to satisfy the microsec wait time requirement. }
  i := 0
  REPEAT WHILE (i < tckCycles)
    pulseClock
    ++ i 

' This program contains a main entry point

'*********************************************
'* Spin code generated by cspin version 0.36 *
'*********************************************
